import logging
from datetime import datetime

import click
import cloup
from rich.live import Live
from rich.spinner import Spinner
from rich.text import Text

from prfiesta import SPINNER_STYLE, __version__
from prfiesta.collectors.github import GitHubCollector
from prfiesta.environment import GitHubEnvironment
from prfiesta.output import output_frame

logger = logging.getLogger(__name__)

github_environment = GitHubEnvironment()

@cloup.command()
@cloup.option_group(
    'general options',
    cloup.option('-u', '--users', required=True, multiple=True, help='The GitHub Users to search for. Can be multiple'),
)
@cloup.option_group(
    'date filter options',
    cloup.option('-a', '--after', type=click.DateTime(formats=['%Y-%m-%d']), help='Only search for pull requests after this date e.g 2023-01-01'),
    cloup.option('-b', '--before', type=click.DateTime(formats=['%Y-%m-%d']), help='Only search for pull requests before this date e.g 2023-04-30'),
    cloup.option('-d', '--use-updated', is_flag=True, default=False, help='filter on when the pr was last updated rather then created'),
)
@cloup.option_group(
    'user filter options',
    cloup.option('-i', '--use-involves', is_flag=True, default=False, help='collect prs where the users are the author or assignee or mentioned or commented'),
    cloup.option('-r', '--use-reviewed-by', is_flag=True, default=False, help='collect prs where the users reviewed them'),
    cloup.option('-rr', '--use-review-requested', is_flag=True, default=False, help='collect prs where the users were requested a review'),
    constraint=cloup.constraints.mutually_exclusive,
    help='Collect alternative details for the users. If omitted then just collect the prs that a user has authored.',
)
@cloup.option_group(
    'output options',
    cloup.option('-o', '--output', default=None, help='The output location'),
    cloup.option('-ot', '--output-type', type=click.Choice(['csv', 'parquet']), default='csv', show_default=True, show_choices=True, help='The output format'),
    cloup.option('-dc', '--drop-columns', multiple=True, help='Drop columns from the output dataframe'),
)
@cloup.option_group(
    'authentication options',
    cloup.option('-x', '--url', help='The URL of the Git provider to use'),
    cloup.option('-t', '--token', help='The Authentication token to use'),
)
@cloup.version_option(__version__)
def main(**kwargs) -> None:

    users: tuple[str] = kwargs.get('users')
    token: str = kwargs.get('token') or github_environment.get_token()
    url: str = kwargs.get('url') or github_environment.get_url()
    output: str = kwargs.get('output')
    output_type: str = kwargs.get('output_type')
    before: datetime = kwargs.get('before')
    after: datetime = kwargs.get('after')
    drop_columns: list[str] = list(kwargs.get('drop_columns'))
    use_updated: bool = kwargs.get('use_updated')
    use_involves: bool = kwargs.get('use_involves')
    use_reviewed_by: bool = kwargs.get('use_reviewed_by')
    use_review_requested: bool = kwargs.get('use_review_requested')

    logger.info('[bold green]PR Fiesta 🦜🥳')

    spinner = Spinner('dots', text=Text('Loading', style=SPINNER_STYLE))

    with Live(spinner, refresh_per_second=20, transient=True):

        collector = GitHubCollector(token=token, url=url, spinner=spinner, drop_columns=drop_columns)
        pr_frame = collector.collect(
            *users,
            after=after,
            before=before,
            use_updated=use_updated,
            use_involves=use_involves,
            use_reviewed_by=use_reviewed_by,
            use_review_requested=use_review_requested)

        if not pr_frame.empty:
            logger.info('Found [bold green]%s[/bold green] pull requests!', pr_frame.shape[0])

            output_frame(pr_frame, output_type, spinner=spinner, output_name=output)
            logger.info('Time to analyze 🔎 See https://github.com/kiran94/prfiesta/blob/main/docs/analysis.md for some inspiration!')

if __name__ == '__main__':  # pragma: nocover
    main()



import importlib.metadata
import logging
import os

from rich.logging import RichHandler

__version__ = importlib.metadata.version(__name__)

LOGGING_LEVEL=os.environ.get('LOGGING_LEVEL', logging.INFO)
LOGGING_FORMAT=os.environ.get('LOGGING_FORMAT', '%(message)s')
SPINNER_STYLE=os.environ.get('SPINNER_STYLE', 'blue')

logging.basicConfig(
    level=LOGGING_LEVEL,
    format=LOGGING_FORMAT,
    handlers=[RichHandler(markup=True, show_path=False, show_time=False, show_level=True)],
)



import calendar
import logging
from typing import Optional, Union

import matplotlib.pyplot as plt
import pandas as pd
import seaborn as sns
from matplotlib.axes import Axes

logger = logging.getLogger(__name__)

_months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December']

def plot_state_distribution(data: pd.DataFrame, **kwargs) -> Union[plt.Figure, plt.Axes, pd.DataFrame]:

    ax: Optional[Axes] = kwargs.get('ax')
    palette: Optional[str] = kwargs.get('palette')
    title: Optional[str] = kwargs.get('title', 'State Distribution')
    hue: Optional[str] = kwargs.get('hue', 'repository_name')

    if ax:
        ax.set_title(title)

    return sns.histplot(data, x='state', hue=hue, ax=ax, palette=palette)

def plot_overall_timeline(data: pd.DataFrame, **kwargs) -> Union[plt.Figure, plt.Axes, pd.DataFrame]:

    ax: Optional[Axes] = kwargs.get('ax')
    palette: Optional[str] = kwargs.get('palette')
    title: Optional[str] = kwargs.get('title', 'Overall Contributions')
    hue: Optional[str] = kwargs.get('hue', 'month')

    temp = data.copy()

    temp['month'] = temp['created_at'].dt.month_name()
    temp['year'] = temp['created_at'].dt.year.astype(str)

    temp = temp.groupby(['month', 'year'])['id'].count()
    temp = temp.reset_index()

    # X Axis Ordering (Year)
    x_order = temp['year'].unique().tolist()
    x_order = [int(x) for x in x_order]
    x_order.sort()
    x_order = [str(x) for x in x_order]

    # Hue Ordering (Months)
    sorted_months = sorted(_months, key=lambda x: list(calendar.month_name).index(x))

    p = sns.barplot(temp, x='year', y='id', hue=hue, ax=ax, order=x_order, hue_order=sorted_months, palette=palette)

    if ax:
        ax.set_title(title)
        ax.legend(loc='upper left')

    return p

def plot_author_associations(data: pd.DataFrame, **kwargs) -> Union[plt.Figure, plt.Axes, pd.DataFrame]:

    ax: Optional[Axes] = kwargs.get('ax')
    palette: Optional[str] = kwargs.get('palette')
    title: Optional[str] = kwargs.get('title', 'Author Associations')

    temp = data.groupby('author_association')['id'].count()
    temp.name = 'count'

    return temp.plot.pie(ax=ax, title=title, legend=True, colormap=palette)

def plot_conventional_commit_breakdown(data: pd.DataFrame, **kwargs) -> Union[plt.Figure, plt.Axes, pd.DataFrame]:

    ax: Optional[Axes] = kwargs.get('ax')
    palette: Optional[str] = kwargs.get('palette')
    title: Optional[str] = kwargs.get('title', 'Conventional Commit Breakdown')
    hue: Optional[str] = kwargs.get('hue', 'type')

    conventional_commit_frame = data['title'] \
        .str \
        .extract(r'^(?P<type>feat|fix|docs|style|refactor|test|chore|build|ci|perf)(\((?P<scope>[A-Za-z-]+)\))?: (?P<subject>[^\n]+)$').copy()

    conventional_commit_frame = conventional_commit_frame.drop(columns=[1, 'subject'])
    conventional_commit_frame = pd.concat([conventional_commit_frame, data['repository_name']], axis=1)
    conventional_commit_frame = conventional_commit_frame.dropna(subset=['type'])

    if conventional_commit_frame.empty:
        logger.warning('passed data did not seem to have any conventional commits')
        return None

    type_count = conventional_commit_frame.groupby(['type', 'repository_name']).count().reset_index()
    type_count = type_count[type_count['scope'] != 0]
    type_count = type_count.sort_values(by='scope', ascending=False)
    type_count = type_count.rename(columns={'scope': 'count'})

    p = sns.barplot(type_count, y='repository_name', x='count', hue=hue, ax=ax, palette=palette)

    if ax:
        ax.set_title(title)
        ax.legend(loc='upper right')

    return p

def plot_reactions(data: pd.DataFrame, **kwargs) -> Union[plt.Figure, plt.Axes, pd.DataFrame]:

    ax: Optional[Axes] = kwargs.get('ax')
    palette: Optional[str] = kwargs.get('palette')
    title: Optional[str] = kwargs.get('title', 'Reactions')
    hue: Optional[str] = kwargs.get('hue')
    threshold: Optional[int] = kwargs.get('threshold')

    reaction_columns = [x for x in data.columns.tolist() if x.startswith('reactions.') and x not in ['reactions.url']]

    reaction_df = data[reaction_columns]
    reaction_df = reaction_df[reaction_df['reactions.total_count'] > 0]
    reaction_df = reaction_df.drop(columns=['reactions.total_count'])

    if reaction_df.empty:
        logger.warning('passed data did not seem to have any reactions 🙁')
        return None

    if threshold:
        reaction_df = reaction_df[reaction_df < threshold]

    p = sns.scatterplot(reaction_df, ax=ax, palette=palette, hue=hue)

    if ax:
        ax.set_title(title)
        ax.legend(loc='upper right')

    return p



import os

from github.Consts import DEFAULT_BASE_URL as GITHUB_DEFAULT_BASE_URL


class GitHubEnvironment:

    def get_token(self) -> str:
        """Gets the authentication token for this environment."""
        token = os.environ.get('GITHUB_ENTERPRISE_TOKEN', os.environ.get('GITHUB_TOKEN'))
        if not token:
            raise ValueError('GITHUB_ENTERPRISE_TOKEN or GITHUB_TOKEN must be set')

        return token

    def get_url(self) -> str:
        """Gets the URL for the git provider."""
        return os.environ.get('GH_HOST', GITHUB_DEFAULT_BASE_URL)



import logging
from datetime import datetime
from typing import Literal

import pandas as pd
from rich.spinner import Spinner

from prfiesta.spinner import update_spinner

logger = logging.getLogger(__name__)

OUTPUT_TYPE = Literal['csv', 'parquet']


def output_frame(
        frame: pd.DataFrame,
        output_type: OUTPUT_TYPE,
        spinner: Spinner,
        output_name: str = None,
        timestamp: datetime = None) -> None:

    if not output_name:
        if not timestamp:
            timestamp = datetime.now()

        output_name = f"export.{timestamp.strftime('%Y-%m-%d_%H:%M:%S')}.{output_type}"

    update_spinner(f'Writing export to {output_name}', spinner, logger)

    if output_type == 'csv':
        frame.to_csv(output_name, index=False)
    elif output_type == 'parquet':
        frame.to_parquet(output_name, index=False)
    else:
        raise ValueError('unknown output_type %s', output_type)

    logger.info('Exported to %s!', output_name)
